/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept X32 execve syscall
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When process calls execve, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 13.VIII.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#ifdef CONFIG_X86_X32

#include "../../../../../p_lkrg_main.h"

#ifdef P_SYSCALL_LAYOUT_4_17


char p_x32_sys_execve_kretprobe_state = 0x0;

static struct kretprobe p_x32_sys_execve_kretprobe = {
    .kp.symbol_name = P_GET_X32_SYSCALL_NAME(execve),
    .handler = p_x32_sys_execve_ret,
    .entry_handler = p_x32_sys_execve_entry,
    .data_size = sizeof(struct p_x32_sys_execve_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_x32_sys_execve_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_x32_sys_execve_entry>\n");

   p_ed_enforce_validation();

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);
   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_execve_entry>\n");

   return 0;
}


int p_x32_sys_execve_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_x32_sys_execve_ret>\n");
/*
   p_inode = p_get_inode_from_task(current);

   p_debug_kprobe_log(
//   p_print_log(P_LKRG_CRIT,
          "p_x32_sys_execve_ret: returned value => %ld comm[%s] Pid:%d inode[%ld]\n",
           p_regs_get_ret(p_regs),current->comm,current->pid,p_inode->i_ino);
*/

   // Update process
   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      if (!p_regs_get_ret(p_regs) ||
          (!task_pid_nr(p_tmp->p_ed_task.p_task) && !p_tmp->p_ed_task.p_task->comm[0])) {
         // This process is on the ED list - update information!
         p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
         p_update_ed_process(p_tmp, current, 1);
         p_reset_ed_flags(p_tmp);
      } else {
         p_set_ed_process_on(p_tmp);
      }
   }
   p_tasks_write_unlock(&p_flags);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_execve_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(x32_sys_execve)

#endif

#endif
