/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept *call_usermodehelper* function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When kernel calls user-mode helper, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.II.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"

#define UMH_LIST                                                \
   {                                                            \
       UMH_UNIFIFW                                              \
       UMH_LNET_UPCALL                                          \
       UMH_OSD_LOGIN                                            \
       { "/bin/false", false },                                 \
       { "/bin/true", false },                                  \
       { "/etc/acpi/events/RadioPower.sh", false },             \
       { "/etc/acpi/wireless-rtl-ac-dc-power.sh", false },      \
       { "/lib/systemd/systemd-cgroups-agent", false },         \
       { "/sbin/bridge-stp", false },                           \
       { "/sbin/critical_overtemp", false },                    \
       { "/sbin/drbdadm", false },                              \
       { "/sbin/hotplug", false },                              \
       { "/sbin/modprobe", false },                             \
       { "/sbin/nfs_cache_getent", false },                     \
       { "/sbin/nfsd-recall-failed", false },                   \
       { "/sbin/nfsdcltrack", false },                          \
       { "/sbin/ocfs2_hb_ctl", false },                         \
       { "/sbin/pnpbios", false },                              \
       { "/sbin/poweroff", false },                             \
       { "/sbin/tomoyo-init", false },                          \
       { "/sbin/v86d", false },                                 \
       { "/system/bin/start", false },                          \
       { "/usr/lib/systemd/systemd-cgroups-agent", false },     \
       { "/usr/lib/systemd/systemd-coredump", false },          \
       { "/usr/sbin/eppfpga", false },                          \
       { "/usr/share/apport/apport", false }                    \
   }

char p_call_usermodehelper_kretprobe_state = 0x0;
static DEFINE_SPINLOCK(p_umh_attr_lock);

static struct kretprobe p_call_usermodehelper_kretprobe = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,3,0)
    .kp.symbol_name = "____call_usermodehelper",
#else
    .kp.symbol_name = "call_usermodehelper_exec_async",
#endif
    .handler = p_call_usermodehelper_ret,
    .entry_handler = p_call_usermodehelper_entry,
    .data_size = sizeof(struct p_call_usermodehelper_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

struct p_umh_path {
   const char *p_path;
   bool p_enabled;
} p_umh_global[] = UMH_LIST;

static int p_check_if_file_exists(const char *p_arg) {

   int p_ret = P_LKRG_SUCCESS;
   struct path p_path;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_check_if_file_exists>\n");
   p_debug_log(P_LKRG_DBG,
          "Trying to resolve [%s]\n",p_arg);

   if ( (p_ret = kern_path(p_arg, LOOKUP_FOLLOW, &p_path)) != P_LKRG_SUCCESS) {
      p_print_log(P_LKRG_INFO,
             "[kern_path] Can't resolve filename: [%s]\n",p_arg);
      p_ret = P_LKRG_GENERAL_ERROR;
      goto p_check_if_file_exists_out;
   }

   path_put(&p_path);

p_check_if_file_exists_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_check_if_file_exists> (p_ret => %d)\n",p_ret);

   return p_ret;
}

void p_usermodehelper_init(void) {

   size_t p_allowed, i;

   for (p_allowed = 0, i = 0; i < nitems(p_umh_global); i++) {
      if (!p_check_if_file_exists(p_umh_global[i].p_path)) {
         p_umh_global[i].p_enabled = true;
         p_print_log(P_LKRG_INFO,
                "Allowing: [%s]\n", p_umh_global[i].p_path);
         p_allowed++;
      }
   }

   p_print_log(P_LKRG_WARN,
          "%zu/%zu UMH paths are allowed...\n",
          p_allowed, nitems(p_umh_global));
}

int p_call_usermodehelper_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   struct subprocess_info *p_subproc = (struct subprocess_info *)p_regs_get_arg1(p_regs);
   unsigned char p_umh_allowed = 0x0;
   unsigned long p_flags;
   struct p_umh_path p_umh_local[] = UMH_LIST;
   size_t i;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_entry>\n");

   p_ed_enforce_validation();

   if (!P_CTRL(p_umh_validate)) {
      goto p_call_usermodehelper_entry_out;
   } else if (P_CTRL(p_umh_validate) == 0x2) {
      goto p_call_usermodehelper_entry_not_allowed;
   }

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   /*
    * Make it harder for the attacker to change the allowed paths by
    * relying on the local variable on the stack
    */
   for (i = 0; i < nitems(p_umh_global); i++) {
      if (p_umh_global[i].p_enabled) {
         if (!strcmp(p_umh_local[i].p_path, p_subproc->path)) {
            p_umh_allowed = 0x1;
            break;
         }
      }
   }

#if defined(CONFIG_STATIC_USERMODEHELPER)
   if (!strcmp(CONFIG_STATIC_USERMODEHELPER_PATH,p_subproc->path)) {
      p_umh_allowed = 0x1;
   }
#endif

   if (!p_umh_allowed) {
p_call_usermodehelper_entry_not_allowed:
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,18,0)
      if (!strcmp("none",p_subproc->path) && p_subproc->file) {
         p_print_log(P_LKRG_ERR,
                "<Exploit Detection> UMH is executing file from memory...\n");
      } else {
#endif
         switch (P_CTRL(p_umh_enforce)) {

            /* Panic */
            case 2:
              panic(P_LKRG_SIGNATURE
                     "Blocked usermodehelper execution of [%s]\n",
                     p_subproc->path);
              break;

            /* Prevent execution */
            case 1:

              i = 0;
              while (p_subproc->path[i] == '/')
                 i++;

              if (i > 1 && !p_subproc->path[i]) { /* at least 2 slashes and nothing else */
                 p_print_log(P_LKRG_CRIT,
                        "Blocked usermodehelper execution of a previously blocked pathname\n");
              } else {
                 spin_lock(&p_umh_attr_lock);
                 p_print_log(P_LKRG_CRIT,
                        "Blocked usermodehelper execution of [%s]\n",
                        p_subproc->path);
                 do {
                    int p_writable = 1;
                    unsigned long p_addr = (unsigned long)p_subproc->path;
                    spinlock_t *p_ptl;
                    pte_t *p_pte;
                    struct mm_struct *p_mm = (current->mm) ? current->mm : current->active_mm;
                    pmd_t *p_pmd = P_SYM(p_mm_find_pmd)(p_mm, p_addr);

                    /*
                     * If we can't find PMD, we assume that the page is writable.
                     * The main reason behind that is because we don't know what
                     * are the current attributes. If we don't know them, we can't
                     * safely modify it because we won't be able to restore them.
                     */
                    if (p_pmd) {
                       p_pte = pte_offset_map_lock(p_mm, p_pmd, p_addr, &p_ptl);
                       if (!pte_write(*p_pte)) {
                          p_writable = 0;
                       }
                       pte_unmap_unlock(p_pte, p_ptl);
                    }

                    if (p_writable) {
                       memset((char *)p_addr, '/', strlen(p_subproc->path));
                    } else {
                       p_set_memory_rw(p_addr & PAGE_MASK,1);
                       memset((char *)p_addr, '/', strlen(p_subproc->path));
                       p_set_memory_ro(p_addr & PAGE_MASK,1);
                    }
                 } while(0);
                 spin_unlock(&p_umh_attr_lock);
              }
              break;

            /* Log only */
            case 0:
              p_print_log(P_LKRG_CRIT,
                     "Detected usermodehelper execution of [%s]\n",
                     p_subproc->path);
              break;

         }
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,18,0)
      }
#endif
   }

p_call_usermodehelper_entry_out:

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_entry>\n");

   return 0;
}


int p_call_usermodehelper_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_ret>\n");

   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(call_usermodehelper)
