/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  On kernel < 4.0 you can't intercept do_init_module function
 *  and each *init_module syscall must be intercepted individually.
 *  This module intercepts sys_finit_module
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.XII.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


//#if LINUX_VERSION_CODE < KERNEL_VERSION(4,0,0)

char p_sys_finit_module_kretprobe_state = 0x0;

static struct kretprobe p_sys_finit_module_kretprobe = {
    .kp.symbol_name = P_GET_SYSCALL_NAME(finit_module),
    .handler = p_sys_finit_module_ret,
    .entry_handler = p_sys_finit_module_entry,
    .data_size = sizeof(struct p_sys_finit_module_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_finit_module_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_entry>\n");

   p_tasks_read_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   p_tasks_read_unlock(&p_flags);

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] is trying to load a kernel module "
             "but does NOT have appropriate permissions! Blocking...\n",
             current->comm,task_pid_nr(current));


      p_syscall_set_arg1(p_regs, -1);
      p_syscall_set_arg2(p_regs, 0x0);

      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_entry>\n");

   return 0x0;
}


int p_sys_finit_module_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   p_debug_kprobe_log(
          "Entering function <p_sys_finit_module_ret>\n");

   p_ed_enforce_validation();
   if (!capable(CAP_SYS_MODULE)) {

      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Process[%s | %d] loaded kernel module but does NOT have "
             "appropriate permissions! Killing...\n",
             current->comm,task_pid_nr(current));

      // kill this process!
      p_ed_kill_task_by_task(current);
   }

   p_debug_kprobe_log(
          "Leaving function <p_sys_finit_module_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(sys_finit_module)

//#endif
