/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept setresgid syscall
 *
 * Notes:
 *  - None
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 09.IX.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_sys_setresgid_kretprobe_state = 0x0;

static struct kretprobe p_sys_setresgid_kretprobe = {
    .kp.symbol_name = P_GET_SET_ID_NAME(setresgid),
    .handler = p_sys_setresgid_ret,
    .entry_handler = p_sys_setresgid_entry,
    .data_size = sizeof(struct p_sys_setresgid_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_setresgid_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_setresgid_entry>\n");

   p_ed_enforce_validation();

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   p_debug_kprobe_log(
          "Leaving function <p_sys_setresgid_entry>\n");

   return 0x0;
}


int p_sys_setresgid_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_setresgid_ret>\n");
   p_debug_kprobe_log(
          "Setresgid returned value => %ld comm[%s] Pid:%d parent[%d]\n",
                       p_regs_get_ret(p_regs),current->comm,current->pid,current->real_parent->pid);

   // Update process
   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
     if (!p_regs_get_ret(p_regs)) {
         // This process is on the ED list - update information!
         p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",current->pid);
         p_update_ed_process(p_tmp, current, 0);
      }
      p_set_ed_process_on(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_sys_setresgid_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(sys_setresgid)
