/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'ttwu_do_wakeup' function
 *
 * Notes:
 *  - Enforce Exploit Detection and pCFI
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 15.III.2020
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_ttwu_do_wakeup_kretprobe_state = 0x0;

static struct kretprobe p_ttwu_do_wakeup_kretprobe = {
    .kp.symbol_name = "ttwu_do_wakeup",
    .handler = p_ttwu_do_wakeup_ret,
    .entry_handler = p_ttwu_do_wakeup_entry,
    .data_size = sizeof(struct p_ttwu_do_wakeup_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_ttwu_do_wakeup_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct task_struct *p_task = (struct task_struct *)p_regs_get_arg2(p_regs);
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_ttwu_do_wakeup_entry>\n");

   p_ed_pcfi_cpu(0);

   if (p_is_ed_task(p_task)) {
      /* Do not take ED lock */
      if (p_tasks_read_trylock(&p_flags)) {
         p_ed_validate_from_running(p_task);
         p_tasks_read_unlock(&p_flags);
      }
   }

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_ttwu_do_wakeup_entry>\n");

   return 0x0;
}


int p_ttwu_do_wakeup_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_ttwu_do_wakeup_ret>\n");


// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_ttwu_do_wakeup_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(ttwu_do_wakeup)
